/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Tom Schindl <tom.schindl@bestsolution.at> - initial API and implementation
 *     Fredy Dobler <fredy@dobler.net> - bug 159600
 *     Brock Janiczak <brockj@tpg.com.au> - bug 182443
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.jface.viewers.ColumnViewerToolTipSupport;

import dwtx.jface.viewers.ColumnViewer;
import dwtx.jface.viewers.ViewerRow;
import dwtx.jface.viewers.ViewerColumn;
import dwtx.jface.viewers.CellLabelProvider;
import dwtx.jface.viewers.StructuredSelection;

import dwt.DWT;
import dwt.graphics.Image;
import dwt.graphics.Point;
import dwt.widgets.Event;
import dwtx.jface.util.Policy;
import dwtx.jface.window.DefaultToolTip;
import dwtx.jface.window.ToolTip;

import dwt.dwthelper.utils;

/**
 * The ColumnViewerTooltipSupport is the class that provides tool tips for ColumnViewers.
 *
 * @since 3.3
 *
 */
public class ColumnViewerToolTipSupport : DefaultToolTip {
    private ColumnViewer viewer;

    private static const String LABEL_PROVIDER_KEY = Policy.JFACE
            ~ "_LABEL_PROVIDER"; //$NON-NLS-1$

    private static const String ELEMENT_KEY = Policy.JFACE ~ "_ELEMENT_KEY"; //$NON-NLS-1$

    private static const int DEFAULT_SHIFT_X = 10;

    private static const int DEFAULT_SHIFT_Y = 0;

    /**
     * Enable ToolTip support for the viewer by creating an instance from this
     * class. To get all necessary informations this support class consults the
     * {@link CellLabelProvider}.
     *
     * @param viewer
     *            the viewer the support is attached to
     * @param style style passed to control tool tip behavior
     *
     * @param manualActivation
     *            <code>true</code> if the activation is done manually using
     *            {@link #show(Point)}
     */
    protected this(ColumnViewer viewer, int style, bool manualActivation ) {
        super(viewer.getControl(),style,manualActivation);
        this.viewer = viewer;
    }

    /**
     * Enable ToolTip support for the viewer by creating an instance from this
     * class. To get all necessary informations this support class consults the
     * {@link CellLabelProvider}.
     *
     * @param viewer
     *            the viewer the support is attached to
     */
    public static void enableFor(ColumnViewer viewer) {
        new ColumnViewerToolTipSupport(viewer,ToolTip.NO_RECREATE,false);
    }

    /**
     * Enable ToolTip support for the viewer by creating an instance from this
     * class. To get all necessary informations this support class consults the
     * {@link CellLabelProvider}.
     *
     * @param viewer
     *            the viewer the support is attached to
     * @param style style passed to control tool tip behavior
     *
     * @see ToolTip#RECREATE
     * @see ToolTip#NO_RECREATE
     */
    public static void enableFor(ColumnViewer viewer, int style) {
        new ColumnViewerToolTipSupport(viewer,style,false);
    }

    protected Object getToolTipArea(Event event) {
        return viewer.getCell(new Point(event.x,event.y));
    }

    protected final bool shouldCreateToolTip(Event event) {
        if( ! super.shouldCreateToolTip(event) ) {
            return false;
        }

        bool rv = false;

        ViewerRow row = viewer.getViewerRow_package(new Point(event.x, event.y));

        viewer.getControl().setToolTipText(""); //$NON-NLS-1$
        Point point = new Point(event.x, event.y);

        if (row !is null) {
            Object element = row.getItem().getData();

            ViewerColumn viewPart = viewer.getViewerColumn(row
                    .getColumnIndex(point));

            if (viewPart is null) {
                return false;
            }

            CellLabelProvider labelProvider = viewPart.getLabelProvider();
            bool useNative = labelProvider.useNativeToolTip(element);

            String text = labelProvider.getToolTipText(element);
            Image img = null;

            if( ! useNative ) {
                img = labelProvider.getToolTipImage(element);
            }

            if( useNative || (text is null && img is null ) ) {
                viewer.getControl().setToolTipText(text);
                rv = false;
            } else {
                setPopupDelay(labelProvider.getToolTipDisplayDelayTime(element));
                setHideDelay(labelProvider.getToolTipTimeDisplayed(element));

                Point shift = labelProvider.getToolTipShift(element);

                if (shift is null) {
                    setShift(new Point(DEFAULT_SHIFT_X, DEFAULT_SHIFT_Y));
                } else {
                    setShift(new Point(shift.x, shift.y));
                }

                setData(LABEL_PROVIDER_KEY, labelProvider);
                setData(ELEMENT_KEY, element);

                setText(text);
                setImage(img);
                setStyle(labelProvider.getToolTipStyle(element));
                setForegroundColor(labelProvider.getToolTipForegroundColor(element));
                setBackgroundColor(labelProvider.getToolTipBackgroundColor(element));
                setFont(labelProvider.getToolTipFont(element));

                // Check if at least one of the values is set
                rv = getText(event) !is null || getImage(event) !is null;
            }
        }

        return rv;
    }

    protected void afterHideToolTip(Event event) {
        if (event !is null && event.widget !is viewer.getControl()) {
            if (event.type is DWT.MouseDown) {
                viewer.setSelection(new StructuredSelection());
            } else {
                viewer.getControl().setFocus();
            }
        }
    }
}
